/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.util;

//---IMPORTS---
import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.io.IOException;

/**
 * Access Control List DatagramSocket wrapper. Used
 * to wrap a DatagramSocket with an ACL, whilst extending
 * DatagramSocket to make integration into code easier.
 * Once the ACL has been set, receive() can be used exactly
 * as it would be with a DatagramSocket.
 *
 * @author  $Author: tdb $
 * @version $Id: ACLDatagramSocket.java,v 1.4 2003/02/05 14:27:58 tdb Exp $
 */
public class ACLDatagramSocket extends DatagramSocket {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.4 $";

//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * See relevant DatagramSocket constructor.
     */
    public ACLDatagramSocket() throws IOException {
        super();
        setACL(new ACL());
    }

    /**
     * See relevant DatagramSocket constructor.
     */
    public ACLDatagramSocket(int port) throws IOException {
        super(port);
        setACL(new ACL());
    }
    
    /**
     * See relevant DatagramSocket constructor.
     */
    public ACLDatagramSocket(int port, InetAddress laddr) throws IOException {
        super(port, laddr);
        setACL(new ACL());
    }
    
    /**
     * See relevant DatagramSocket constructor, and
     * sets the default acl.
     */
    public ACLDatagramSocket(ACL acl) throws IOException {
        super();
        setACL(acl);
    }
        
    /**
     * See relevant DatagramSocket constructor, and
     * sets the default acl.
     */
    public ACLDatagramSocket(ACL acl, int port) throws IOException {
        super(port);
        setACL(acl);
    }
    
    /**
     * See relevant DatagramSocket constructor, and
     * sets the default acl.
     */
    public ACLDatagramSocket(ACL acl, int port, InetAddress laddr) throws IOException {
        super(port, laddr);
        setACL(acl);
    }

//---PUBLIC METHODS---

    /**
     * Essentially has the same behaviour as the
     * Datagram.receive() method, except won't
     * return  unless it is permitted by the ACL.
     */
    public void receive(DatagramPacket p) throws IOException {
        while(true) {
            super.receive(p);
            if(_acl.check(p.getAddress())) {
                return;
            }
        }
    }
    
    /**
     * Set the ACL at any point in the operation
     * of the DatagramSocket.
     *
     * @param acl the new ACL to apply
     */
    public void setACL(ACL acl) {
        _acl = acl;
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.server.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),            
            REVISION);
            // super.toString() isn't implemented...
            //REVISION) + ":" + super.toString();
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = null;
    
    /**
     * The ACL used by this ACLDatagramSocket.
     */
    private ACL _acl;
    
//---STATIC ATTRIBUTES---

//---INNER CLASSES---

}
